-------------------------------------------------------------------------------
-- Project     : HACM
-- Author      : Alexandru Gheolbanoiu, Lucian Petrica
-- Date        : 31.08.2014
-- File        : hacm_core.vhd
-- Description :
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
library unisim;
use unisim.vcomponents.all;

-- ENTITY ---------------------------------------------------------------------
entity hacm_core is
	port
	(
		clk_i     : in  std_logic;
        rst_i     : in std_logic;
		mode_i    : in  std_logic; --0=CS; 1=CM
		stretch_i : in  std_logic_vector(1 downto 0);
		sel_i     : in  std_logic; --0=PC; 1=CO
		lock_o    : out std_logic;
		clk_o     : out std_logic;
        --dynamic reconfiguration interface
        drp_addr  : in std_logic_vector(6 downto 0);
        drp_din   : in std_logic_vector(15 downto 0);
        drp_dout  : out std_logic_vector(15 downto 0);
        drp_we    : in std_logic;
        drp_en    : in std_logic;
        drp_rdy   : out std_logic;
        drp_clk   : in std_logic;
        --phase shift interface
        ps_en     : in std_logic;
        ps_incdec : in std_logic;
        ps_done   : out std_logic;
        ps_clk    : in std_logic
	);
end hacm_core;

-- ARCHITECTURE ---------------------------------------------------------------
architecture rtl of hacm_core is

  --Multiplexing Signals
  signal clk0 : std_logic;
  signal clk1 : std_logic;
  signal clk2 : std_logic;
  signal clk3 : std_logic;
  signal clk4 : std_logic;
  signal clkout: std_logic;

  signal clk00 : std_logic;
  signal clk01 : std_logic;
  signal clk10 : std_logic;
  signal clk11 : std_logic;

  --MMCM Signals and Attributes
  attribute CORE_GENERATION_INFO : string;
  attribute CORE_GENERATION_INFO of rtl : architecture is "clk_wiz_v3_6,clk_wiz_v3_6,{component_name=clk_wiz_v3_6,use_phase_alignment=false,use_min_o_jitter=false,use_max_i_jitter=false,use_dyn_phase_shift=false,use_inclk_switchover=false,use_dyn_reconfig=true,feedback_source=FDBK_AUTO,primtype_sel=MMCM_ADV,num_out_clk=6,clkin1_period=10.0,clkin2_period=10.0,use_power_down=false,use_reset=true,use_locked=true,use_inclk_stopped=false,use_status=false,use_freeze=false,use_clk_valid=false,feedback_type=SINGLE,clock_mgr_type=MANUAL,manual_override=false}";
  signal clkin            : std_logic;
  signal clkfb            : std_logic;
  signal clkout0          : std_logic;
  signal clkout1          : std_logic;
  signal clkout2          : std_logic;
  signal clkout3          : std_logic;
  signal clkout4          : std_logic;

  --Clock Control
  signal clk_mux_sel_reg_s    : std_logic_vector(1 downto 0) := "00";
  signal clk_mux_sel_reg_del_s: std_logic_vector(1 downto 0) := "00";
  signal cm_mux_sel_reg_s     : std_logic := '0';

begin
  --Clock Mux Control
  process(clkout)
  begin
    if(clkout'event and clkout='1') then
	   if(mode_i = '1') then
		  clk_mux_sel_reg_del_s <= "00";
	     clk_mux_sel_reg_s     <= "00";
		  cm_mux_sel_reg_s      <= sel_i;
		else
	     clk_mux_sel_reg_del_s <= clk_mux_sel_reg_s;
	     clk_mux_sel_reg_s     <= clk_mux_sel_reg_s + stretch_i;
		  cm_mux_sel_reg_s      <= '0';
      end if;
	 end if;
  end process;


  --Clock Multiplexing
  ---Layer1
  mux00 : BUFGCTRL generic map ( INIT_OUT => 0, PRESELECT_I0 => FALSE, PRESELECT_I1 => FALSE)
  port map (O => clk00,  I0 => clk0, I1 => clk1,
	         IGNORE0 => clk_mux_sel_reg_del_s(1), IGNORE1 => clk_mux_sel_reg_del_s(1),
				CE0 => '1', CE1 => '1',
				S0 => not(clk_mux_sel_reg_s(0)), S1 => clk_mux_sel_reg_s(0) );
  mux01 : BUFGCTRL generic map ( INIT_OUT => 0, PRESELECT_I0 => FALSE, PRESELECT_I1 => FALSE)
  port map (O => clk01, I0 => clk2, I1 => clk3,
	         IGNORE0 => not(clk_mux_sel_reg_del_s(1)), IGNORE1 => not(clk_mux_sel_reg_del_s(1)),
				CE0 => '1', CE1 => '1',
				S0 => not(clk_mux_sel_reg_s(0)), S1 => clk_mux_sel_reg_s(0) );
  ---Layer2
  mux10 : BUFGCTRL generic map ( INIT_OUT => 0, PRESELECT_I0 => FALSE, PRESELECT_I1 => FALSE)
  port map (O => clk10, I0 => clk00, I1 => clk4,
	         IGNORE0 => '0', IGNORE1 => '0',
				CE0 => '1', CE1 => '1',
				S0 => not(cm_mux_sel_reg_s), S1 => cm_mux_sel_reg_s );
  mux11 : BUFGCTRL generic map ( INIT_OUT => 0, PRESELECT_I0 => FALSE, PRESELECT_I1 => FALSE)
  port map (O => clk11, I0 => clk01, I1 => '0',
	         IGNORE0 => '0', IGNORE1 => '0',
				CE0 => '1', CE1 => '1',
				S0 => not(cm_mux_sel_reg_s), S1 => cm_mux_sel_reg_s);
  ---Layer3
  mux20 : BUFGCTRL generic map ( INIT_OUT => 0, PRESELECT_I0 => FALSE, PRESELECT_I1 => FALSE)
  port map (O => clkout,  I0 => clk10, I1 => clk11,
	         IGNORE0 => '0', IGNORE1 => '0',
            CE0 => '1', CE1 => '1',
				S0 => not(clk_mux_sel_reg_s(1)), S1 => clk_mux_sel_reg_s(1) );

  --MMCM & Buffers
  clkin <= clk_i;
  clk0  <= clkout0;
  clk1  <= clkout1;
  clk2  <= clkout2;
  clk3  <= clkout3;
  clk4  <= clkout4;
  clk_o <= clkout;

  mmcm_adv_inst : MMCME2_ADV
  generic map
   (BANDWIDTH            => "OPTIMIZED",
    CLKOUT4_CASCADE      => FALSE,
    COMPENSATION         => "ZHOLD",
    STARTUP_WAIT         => FALSE,
    DIVCLK_DIVIDE        => 1,
    CLKFBOUT_MULT_F      => 10.000,
    CLKFBOUT_PHASE       => 0.000,
    CLKFBOUT_USE_FINE_PS => FALSE,
    CLKOUT0_DIVIDE_F     => 10.000,
    CLKOUT0_PHASE        => 0.000,
    CLKOUT0_DUTY_CYCLE   => 0.500,
    CLKOUT0_USE_FINE_PS  => FALSE,
    CLKOUT1_DIVIDE       => 10,
    CLKOUT1_PHASE        => 90.000,
    CLKOUT1_DUTY_CYCLE   => 0.500,
    CLKOUT1_USE_FINE_PS  => FALSE,
    CLKOUT2_DIVIDE       => 10,
    CLKOUT2_PHASE        => 180.000,
    CLKOUT2_DUTY_CYCLE   => 0.500,
    CLKOUT2_USE_FINE_PS  => FALSE,
    CLKOUT3_DIVIDE       => 10,
    CLKOUT3_PHASE        => 270.000,
    CLKOUT3_DUTY_CYCLE   => 0.500,
    CLKOUT3_USE_FINE_PS  => FALSE,
	CLKOUT4_DIVIDE       => 11,
    CLKOUT4_PHASE        => 0.000,
    CLKOUT4_DUTY_CYCLE   => 0.500,
    CLKOUT4_USE_FINE_PS  => FALSE,
    CLKIN1_PERIOD        => 10.000,
    REF_JITTER1          => 0.010)
  port map
    -- Output clocks
   (CLKFBOUT            => clkfb,
    CLKFBOUTB           => open,
    CLKOUT0             => clkout0,
    CLKOUT0B            => open,
    CLKOUT1             => clkout1,
    CLKOUT1B            => open,
    CLKOUT2             => clkout2,
    CLKOUT2B            => open,
    CLKOUT3             => clkout3,
    CLKOUT3B            => open,
    CLKOUT4             => clkout4,
    CLKOUT5             => open,
    CLKOUT6             => open,
    -- Input clock control
    CLKFBIN             => clkfb,
    CLKIN1              => clkin,
    CLKIN2              => '0',
    -- Tied to always select the primary input clock
    CLKINSEL            => '1',
    -- Ports for dynamic reconfiguration
    DADDR               => drp_addr,
    DCLK                => drp_clk,
    DEN                 => drp_en,
    DI                  => drp_din,
    DO                  => drp_dout,
    DRDY                => drp_rdy,
    DWE                 => drp_we,
    -- Ports for dynamic phase shift
    PSCLK               => ps_clk,
    PSEN                => ps_en,
    PSINCDEC            => ps_incdec,
    PSDONE              => ps_done,
    -- Other control and status signals
    LOCKED              => lock_o,
    CLKINSTOPPED        => open,
    CLKFBSTOPPED        => open,
    PWRDWN              => '0',
    RST                 => rst_i);
-------------------------------------------------------------------------------
end rtl;
