`timescale 1ns / 1ps

module tb;
    localparam ADDR_WIDTH = 10;
    localparam DATA_WIDTH = 32;

    reg clock;
    reg reset_n;

    // AXI4-Lite Interface
    // Write address channel
    wire                    axi4l_awvalid;
    wire                    axi4l_awready;
    wire   [ADDR_WIDTH-1:0] axi4l_awaddr;
    wire              [2:0] axi4l_awprot;
    // Write data channel
    wire                    axi4l_wvalid;
    wire                    axi4l_wready;
    wire [DATA_WIDTH  -1:0] axi4l_wdata;
    wire [DATA_WIDTH/8-1:0] axi4l_wstrb;
    // Write response channel
    wire                    axi4l_bvalid;
    wire                    axi4l_bready;
    wire              [1:0] axi4l_bresp; // Note: 00 = OKAY, 10 = SLVERR (write error)
    // Read address channel
    wire                    axi4l_arvalid;
    wire                    axi4l_arready;
    wire   [ADDR_WIDTH-1:0] axi4l_araddr;
    wire              [2:0] axi4l_arprot;
    // Read data channel
    wire                    axi4l_rvalid;
    wire                    axi4l_rready;
    wire              [1:0] axi4l_rresp; // Note: 00 = OKAY, 10 = SLVERR (read error)
    wire   [DATA_WIDTH-1:0] axi4l_rdata;

    // HACM Inputs
    reg f_or_s_i = 0;

    // HACM Outputs
    wire lock_o;
    wire clk_o;

    // AXI4-Lite Master BFM
    wire done;

    initial begin
        clock = 0;
        forever #5 clock = ~clock;
    end

    initial begin
        reset_n = 1;
        #2   reset_n = ~reset_n;
        #118 reset_n = ~reset_n;
    end

    always @(posedge done)
        $finish;

    initial begin
        #10000;
        forever begin
            f_or_s_i <= ~f_or_s_i;

            if ($random % 10 < 4)
                repeat (($random % 2) + 1)
                    @(posedge clock);
            else
                repeat (($random % 200) + 1)
                    @(posedge clock);

            f_or_s_i <= ~f_or_s_i;

            repeat (($random % 2000) + 1)
                @(posedge clock);
        end
    end

    initial
        forever @(f_or_s_i) begin: clock_check
            realtime start;
            realtime finish;

            repeat (10)
                @(posedge clk_o);

            start = $realtime;
            @(posedge clk_o);
            finish = $realtime;

            if (finish - start != 10.0 + 3.0 * f_or_s_i) begin
                $display("@%0.3f: incorrect clk_o period %0.3fns", $realtime, finish - start);
                $stop;
            end
        end

    axi4lite_master_bfm #(
        .ADDR_WIDTH(ADDR_WIDTH),
        .COMMAND_FILE("cmd.txt")
    ) axi4l_master (
        .axi4l_aclk   (clock        ),
        .axi4l_aresetn(reset_n      ),
        .axi4l_awready(axi4l_awready),
        .axi4l_awvalid(axi4l_awvalid),
        .axi4l_awaddr (axi4l_awaddr ),
        .axi4l_awprot (axi4l_awprot ),
        .axi4l_wvalid (axi4l_wvalid ),
        .axi4l_wready (axi4l_wready ),
        .axi4l_wdata  (axi4l_wdata  ),
        .axi4l_wstrb  (axi4l_wstrb  ),
        .axi4l_bvalid (axi4l_bvalid ),
        .axi4l_bready (axi4l_bready ),
        .axi4l_bresp  (axi4l_bresp  ),
        .axi4l_arvalid(axi4l_arvalid),
        .axi4l_arready(axi4l_arready),
        .axi4l_araddr (axi4l_araddr ),
        .axi4l_arprot (axi4l_arprot ),
        .axi4l_rvalid (axi4l_rvalid ),
        .axi4l_rready (axi4l_rready ),
        .axi4l_rresp  (axi4l_rresp  ),
        .axi4l_rdata  (axi4l_rdata  ),
        .done         (done         )
    );

    axi_hacm #(
        .ADDR_WIDTH(ADDR_WIDTH),
        .DATA_WIDTH(DATA_WIDTH)
    ) hacm_wrapper_inst(
        .axi4l_aclk   (clock        ),
        .axi4l_aresetn(reset_n      ),
        .axi4l_awvalid(axi4l_awvalid),
        .axi4l_awready(axi4l_awready),
        .axi4l_awaddr (axi4l_awaddr ),
        .axi4l_awprot (axi4l_awprot ),
        .axi4l_wvalid (axi4l_wvalid ),
        .axi4l_wready (axi4l_wready ),
        .axi4l_wdata  (axi4l_wdata  ),
        .axi4l_wstrb  (axi4l_wstrb  ),
        .axi4l_bvalid (axi4l_bvalid ),
        .axi4l_bready (axi4l_bready ),
        .axi4l_bresp  (axi4l_bresp  ),
        .axi4l_arvalid(axi4l_arvalid),
        .axi4l_arready(axi4l_arready),
        .axi4l_araddr (axi4l_araddr ),
        .axi4l_arprot (axi4l_arprot ),
        .axi4l_rvalid (axi4l_rvalid ),
        .axi4l_rready (axi4l_rready ),
        .axi4l_rresp  (axi4l_rresp  ),
        .axi4l_rdata  (axi4l_rdata  ),
        .f_or_s_i     (f_or_s_i     ),
        .lock_o       (lock_o       ),
        .clk_o        (clk_o        )
    );
endmodule
